@@ -29,8 +29,6 @@ __authors__ = ["J. Garriga"]
 __license__ = "MIT"
 __date__ = "16/12/2019"
 
-import silx.resources
-
 
 class Config(object):
     """
@@ -40,6 +38,7 @@ class Config(object):
     """
 
     DEFAULT_COLORMAP_NAME = 'cividis'
+    DEFAULT_COLORMAP_NORM = 'log'
     """Default LUT for the plot widgets.
 
     The available list of names are available in the module
@@ -47,6 +46,3 @@ class Config(object):
 
     .. versionadded:: 0.3
     """
-
-    def __init__(self):
-        silx.resources.register_resource_directory("darfix", "darfix.resources")
