@@ -26,30 +26,29 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "10/09/2019"
+__date__ = "04/03/2020"
 
 
+import enum
 import numpy
 import cv2
 from scipy.ndimage import center_of_mass, fourier_shift
 from skimage import feature
-from enum import Enum
+
+from silx.utils.enum import Enum as _Enum
 
 from darfix.io import utils
 from.autofocus import normalized_variance
 
 
-class ShiftApproaches(Enum):
+@enum.unique
+class ShiftApproach(_Enum):
     """
     Different shifts approaches that can be used for the shift detection and correction.
     """
     LINEAR = "linear"
     FFT = "fft"
 
-    @staticmethod
-    def list():
-        return list(map(lambda c: c.value, ShiftApproaches))
-
 
 def compute_com(data):
     """
@@ -73,7 +72,7 @@ def diff_com(img1, img2):
     return numpy.array(center_of_mass(img2)) - numpy.array(center_of_mass(img1))
 
 
-def find_shift(img1, img2, upsampling_factor=1):
+def find_shift(img1, img2, upsampling_factor=1000):
     """
     Uses the function ``register_translation`` from skimage to find the shift between two images.
 
@@ -142,7 +141,7 @@ def _scipy_fft_shift(img, dx, dy):
     return numpy.fft.ifftn(fourier_shift(numpy.fft.fftn(img), [dy, dx])).real
 
 
-def apply_shift(img, shift, shift_approach='fft'):
+def apply_shift(img, shift, shift_approach="fft"):
     """
     Function to apply the shift to an image.
 
@@ -152,9 +151,8 @@ def apply_shift(img, shift, shift_approach='fft'):
     :param Union[`linear`, `fft`] shift_approach: The shift method to be used to apply the shift.
     :returns: real ndarray
     """
-    assert shift_approach in ShiftApproaches.list(), utils.assert_string("Attribute shift_approach",
-                                                                         ShiftApproaches.list())
-    if shift_approach == ShiftApproaches.LINEAR.value:
+    shift_approach = ShiftApproach.from_value(shift_approach)
+    if shift_approach == ShiftApproach.LINEAR:
         img = img.astype(numpy.int16)
         M = numpy.float32([[1, 0, shift[1]], [0, 1, shift[0]]])
         return cv2.warpAffine(img, M, (img.shape[1], img.shape[0]))
@@ -171,7 +169,7 @@ def normalize(x):
     return x / numpy.linalg.norm(x)
 
 
-def improve_linear_shift(data, v, h_max, h_step, nimages=None, shift_approach='linear'):
+def improve_linear_shift(data, v, h_max, h_step, nimages=None, shift_approach="linear"):
     """
     Function to find the best shift between the images. It loops ``h_max * h_step`` times,
     applying a different shift each, and trying to find the one that has the best result.
@@ -189,6 +187,7 @@ def improve_linear_shift(data, v, h_max, h_step, nimages=None, shift_approach='l
     :param Union[`linear`,`fft`] shift_approach: The shift method to be used to apply the shift.
     :returns: ndarray
     """
+    shift_approach = ShiftApproach.from_value(shift_approach)
     v = numpy.asanyarray(v)
     iData = range(data.shape[0])
 
@@ -221,16 +220,29 @@ def shift_detection(data, h_max=0.5, h_step=0.01):
         to apply to every image.
     :rtype: ndarray
     """
-    intensity, com = compute_com(data)
-    first_sum = numpy.sum(data[:com], axis=0)
-    second_sum = numpy.sum(data[com:], axis=0)
+    if len(data) == 0:
+        return []
+    shape = data[0].shape
+    first_sum = numpy.zeros(shape)
+    second_sum = numpy.zeros(shape)
+    for i in range(len(data)):
+        if i < len(data) / 2:
+            first_sum += data[i]
+        else:
+            second_sum += data[i]
     shift = find_shift(first_sum, second_sum, 1000)
     v = normalize(shift)
     h = improve_linear_shift(data, v, h_max, h_step)
-    return numpy.outer(h * v, numpy.arange(data.shape[0]))
+    return numpy.outer(h * v, numpy.arange(len(data)))
 
 
-def shift_correction(data, n_shift, shift_approach='fft', callback=None):
+def sum_images(image1, image2=None):
+    if image2 is None:
+        return numpy.asarray(image1)
+    return numpy.asarray(image1) + numpy.asarray(image2)
+
+
+def shift_correction(data, n_shift, shift_approach="fft", callback=None):
     """
     Function to apply shift correction technique to stack of images.
 
@@ -246,6 +258,7 @@ def shift_correction(data, n_shift, shift_approach='fft', callback=None):
     :returns: The shifted images.
     :rtype: ndarray
     """
+    shift_approach = ShiftApproach.from_value(shift_approach)
 
     shift = numpy.asanyarray(n_shift)
 
@@ -305,7 +318,8 @@ def random_search(data, optimal_shift, iterations, sigma=None, shift_approach='l
         for iFrame in range(len(data)):
             shifty = n_shift[0][iFrame]
             shiftx = n_shift[1][iFrame]
-            result += apply_shift(data[iFrame], [shifty, shiftx], shift_approach)
+            result += data.apply_funcs([(apply_shift, [[shifty, shiftx], shift_approach])],
+                                       indices=iFrame, return_data=True)[0]
         score = normalized_variance(result)
         if best_score < score:
             best_result = n_shift
