@@ -33,6 +33,7 @@ from silx.gui.colors import Colormap
 from silx.gui.plot.StackView import StackViewMainWindow
 
 import darfix
+from darfix.core.dataset import Operation
 from darfix.core.imageOperations import Method
 
 from .operationThread import OperationThread
@@ -46,7 +47,7 @@ class _ParametersDock(qt.QDockWidget):
         """
         qt.QDockWidget.__init__(self, parent)
         widget = qt.QWidget()
-        layout = qt.QGridLayout()
+        self.layout = qt.QGridLayout()
 
         # Background subtraction
         bsLabel = qt.QLabel("Background Substraction")
@@ -54,33 +55,51 @@ class _ParametersDock(qt.QDockWidget):
         font = qt.QFont()
         font.setBold(True)
         bsLabel.setFont(font)
-        layout.addWidget(bsLabel, 0, 0, 1, 2)
+        self.layout.addWidget(bsLabel, 0, 0, 1, 2)
         self.bsMethodsCB = qt.QComboBox(self)
         for method in Method.values():
             self.bsMethodsCB.addItem(method)
         self.bsBackgroundCB = qt.QComboBox(self)
         self.computeBS = qt.QPushButton("Compute")
+        self.abortBS = qt.QPushButton("Abort")
+        self.abortBS.hide()
         methodLabel = qt.QLabel("Method:")
         bgLabel = qt.QLabel("Background:")
-        layout.addWidget(methodLabel, 1, 0, 1, 0.7)
-        layout.addWidget(bgLabel, 2, 0, 1, 0.7)
-        layout.addWidget(self.bsMethodsCB, 1, 1, 1, 1)
-        layout.addWidget(self.bsBackgroundCB, 2, 1, 1, 1)
-        layout.addWidget(self.computeBS, 3, 1, 1, 1)
+        self.chunksWidget = qt.QWidget()
+        chunksLayout = qt.QHBoxLayout()
+        chunksLabel = qt.QLabel("Chunks:")
+        self.verticalChunkSize = qt.QLineEdit("100")
+        self.verticalChunkSize.setValidator(qt.QIntValidator())
+        self.horizontalChunkSize = qt.QLineEdit("100")
+        self.horizontalChunkSize.setValidator(qt.QIntValidator())
+        chunksLayout.addWidget(chunksLabel)
+        chunksLayout.addWidget(self.verticalChunkSize)
+        chunksLayout.addWidget(self.horizontalChunkSize)
+        chunksLayout.setContentsMargins(0, 0, 1, 1)
+        self.chunksWidget.setLayout(chunksLayout)
+        self.layout.addWidget(methodLabel, 1, 0, 1, 0.7)
+        self.layout.addWidget(bgLabel, 2, 0, 1, 0.7)
+        self.layout.addWidget(self.bsMethodsCB, 1, 1, 1, 1)
+        self.layout.addWidget(self.bsBackgroundCB, 2, 1, 1, 1)
+        self.layout.addWidget(self.chunksWidget, 3, 0, 1, 2)
+        self.chunksWidget.hide()
         # Hot pixel removal
         hpLabel = qt.QLabel("Hot Pixel Removal")
         hpLabel.setFont(font)
-        layout.addWidget(hpLabel, 0, 2, 1, 2)
+        self.layout.addWidget(hpLabel, 0, 2, 1, 2)
         ksizeLabel = qt.QLabel("Kernel size:")
-        layout.addWidget(ksizeLabel, 1, 2, 1, 1)
+        self.layout.addWidget(ksizeLabel, 1, 2, 1, 1)
         self.hpSizeCB = qt.QComboBox(self)
         self.hpSizeCB.addItem("3")
         self.hpSizeCB.addItem("5")
         self.computeHP = qt.QPushButton("Compute")
-        layout.addWidget(self.hpSizeCB, 1, 3, 1, 1)
-        layout.addWidget(self.computeHP, 3, 3, 1, 1)
+        self.abortHP = qt.QPushButton("Abort")
+        self.layout.addWidget(self.hpSizeCB, 1, 3, 1, 1)
+        self.layout.addWidget(self.computeHP, 3, 3, 1, 1)
+        self.layout.addWidget(self.abortHP, 3, 2, 1, 1)
+        self.abortHP.hide()
 
-        widget.setLayout(layout)
+        widget.setLayout(self.layout)
         self.setWidget(widget)
 
 
@@ -130,13 +149,13 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._parametersDock.setFeatures(qt.QDockWidget.DockWidgetMovable)
         self._sv = StackViewMainWindow()
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
-                                      normalization='linear',
-                                      vmin=None,
-                                      vmax=None))
+                                      normalization=darfix.config.DEFAULT_COLORMAP_NORM))
         self.setCentralWidget(self._sv)
         self.addDockWidget(qt.Qt.BottomDockWidgetArea, self._parametersDock)
         self._parametersDock.computeBS.clicked.connect(self.__computeBS)
         self._parametersDock.computeHP.clicked.connect(self.__computeHP)
+        self._parametersDock.abortBS.clicked.connect(self.__abortBS)
+        self._parametersDock.abortHP.clicked.connect(self.__abortHP)
 
     def __computeBS(self):
         """
@@ -145,14 +164,19 @@ class NoiseRemovalWidget(qt.QMainWindow):
         """
         method = self._parametersDock.bsMethodsCB.currentText()
         background = self._parametersDock.bsBackgroundCB.currentText()
-        self._thread = OperationThread(self._update_dataset.apply_background_subtraction)
+        self._thread = OperationThread(self, self._update_dataset.apply_background_subtraction)
         bg = None
+        chunks = None
         if background == "Dark data":
             bg = self.bg_dataset
         elif background == "Low intensity data":
             bg = self.bg_indices
-        self._thread.setArgs(bg, method, self.indices)
+        if not self._update_dataset.in_memory:
+            chunks = [int(self._parametersDock.verticalChunkSize.text()),
+                      int(self._parametersDock.horizontalChunkSize.text())]
+        self._thread.setArgs(bg, method, self.indices, chunks)
         self._thread.finished.connect(self._updateData)
+        self._parametersDock.abortBS.show()
         self._thread.start()
         self._parametersDock.computeBS.setEnabled(False)
         self._parametersDock.computeHP.setEnabled(False)
@@ -163,22 +187,38 @@ class NoiseRemovalWidget(qt.QMainWindow):
         removal.
         """
         size = self._parametersDock.hpSizeCB.currentText()
-        self._thread = OperationThread(self.dataset.apply_hot_pixel_removal)
+        self._thread = OperationThread(self, self._update_dataset.apply_hot_pixel_removal)
         self._thread.setArgs(int(size), self.indices)
         self._thread.finished.connect(self._updateData)
+        self._parametersDock.abortHP.show()
         self._thread.start()
         self._parametersDock.computeBS.setEnabled(False)
         self._parametersDock.computeHP.setEnabled(False)
 
+    def __abortBS(self):
+        self._parametersDock.abortBS.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.BS)
+
+    def __abortHP(self):
+        self._parametersDock.abortHP.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.HP)
+
     def _updateData(self):
         """
         Updates the stack with the data computed in the thread
         """
         self._thread.finished.disconnect(self._updateData)
+        self._parametersDock.abortBS.setEnabled(True)
+        self._parametersDock.abortHP.setEnabled(True)
+        self._parametersDock.abortBS.hide()
+        self._parametersDock.abortHP.hide()
         self._parametersDock.computeBS.setEnabled(True)
         self._parametersDock.computeHP.setEnabled(True)
-        self._update_dataset = self._thread.data
-        self.setStack(self._update_dataset)
+        if self._thread.data is not None:
+            self._update_dataset = self._thread.data
+            self.setStack(self._update_dataset)
+        else:
+            print("\nComputation aborted")
 
     def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
@@ -193,6 +233,15 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self.setStack()
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
+        if not dataset.in_memory:
+            self._parametersDock.chunksWidget.show()
+            self._parametersDock.layout.addWidget(self._parametersDock.abortBS, 4, 0, 1, 1)
+            self._parametersDock.layout.addWidget(self._parametersDock.computeBS, 4, 1, 1, 1)
+        else:
+            self._parametersDock.chunksWidget.hide()
+            self._parametersDock.layout.addWidget(self._parametersDock.abortBS, 3, 0, 1, 1)
+            self._parametersDock.layout.addWidget(self._parametersDock.computeBS, 3, 1, 1, 1)
+
         """
         Sets the available background for the user to choose.
         """
