@@ -72,9 +72,7 @@ class ROISelectionWidget(qt.QWidget):
         self._resetB.clicked.connect(self.resetStack)
 
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
-                                      normalization='linear',
-                                      vmin=None,
-                                      vmax=None))
+                                      normalization=darfix.config.DEFAULT_COLORMAP_NORM))
         self.layout().addWidget(self._sv)
         self.layout().addWidget(_buttons)
 
@@ -215,6 +213,5 @@ class ROISelectionWidget(qt.QWidget):
         """
         Clears stack.
         """
-        self._okB.setEnabled(False)
         self._sv.setStack(None)
         self._roi.setGeometry(origin=(0, 0), size=(10, 10))
