@@ -36,6 +36,7 @@ from silx.gui.colors import Colormap
 from silx.gui.plot.StackView import StackViewMainWindow
 
 import darfix
+from darfix.core.dataset import Operation
 
 from .operationThread import OperationThread
 from .utils import ChooseDimensionDock
@@ -64,9 +65,7 @@ class ShiftCorrectionWidget(qt.QMainWindow):
 
         self._sv = StackViewMainWindow()
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
-                                      normalization='linear',
-                                      vmin=None,
-                                      vmax=None))
+                                      normalization=darfix.config.DEFAULT_COLORMAP_NORM))
         self.setCentralWidget(self._sv)
         self._chooseDimensionDock = ChooseDimensionDock(self)
         spacer1 = qt.QWidget(parent=self)
@@ -82,6 +81,7 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self.addDockWidget(qt.Qt.RightDockWidgetArea, self._inputDock)
 
         self._inputDock.widget.correctionB.clicked.connect(self.correct)
+        self._inputDock.widget.abortB.clicked.connect(self.abort)
         self._inputDock.widget._findShiftB.clicked.connect(self._findShift)
         self._inputDock.widget.checkbox.stateChanged.connect(self._changeDataShiftFlag)
         self._chooseDimensionDock.widget.filterChanged.connect(self._filterStack)
@@ -118,12 +118,17 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self.shift = [dy, dx]
         dimension = self.dimension if not self._changeDataShiftFlag else None
         frames = numpy.arange(self.dataset.get_data(indices=self.indices, dimension=dimension).shape[0])
-        self.thread_correction = OperationThread(self._update_dataset.apply_shift)
+        self.thread_correction = OperationThread(self, self._update_dataset.apply_shift)
         self.thread_correction.setArgs(numpy.outer(self.shift, frames), dimension, indices=self.indices)
         self.thread_correction.finished.connect(self._updateData)
         self._inputDock.widget.correctionB.setEnabled(False)
+        self._inputDock.widget.abortB.show()
         self.thread_correction.start()
 
+    def abort(self):
+        self._inputDock.widget.abortB.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.SHIFT)
+
     def updateProgress(self, progress):
         self.sigProgressChanged.emit(progress)
 
@@ -131,7 +136,7 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self._shift2dataset = state
 
     def _findShift(self):
-        self.thread_detection = OperationThread(self._update_dataset.find_shift)
+        self.thread_detection = OperationThread(self, self._update_dataset.find_shift)
         self._inputDock.widget._findShiftB.setEnabled(False)
         self.thread_detection.setArgs(self.dimension, indices=self.indices)
         self.thread_detection.finished.connect(self._updateShift)
@@ -147,11 +152,16 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         Updates the stack with the data computed in the thread
         """
         self.thread_correction.finished.disconnect(self._updateData)
+        self._inputDock.widget.abortB.hide()
+        self._inputDock.widget.abortB.setEnabled(True)
         self._inputDock.widget.correctionB.setEnabled(True)
-        self._update_dataset = self.thread_correction.data
-        assert self._update_dataset is not None
-        self.setStack(self._update_dataset)
-        self.sigComputed.emit()
+        if self.thread_correction.data:
+            self._update_dataset = self.thread_correction.data
+            assert self._update_dataset is not None
+            self.setStack(self._update_dataset)
+            self.sigComputed.emit()
+        else:
+            print("\nCorrection aborted")
 
     def setStack(self, dataset=None):
         """
@@ -227,6 +237,8 @@ class _InputWidget(qt.QWidget):
         self.dxLE = qt.QLineEdit("0.0")
         self.dyLE = qt.QLineEdit("0.0")
         self.correctionB = qt.QPushButton("Correct")
+        self.abortB = qt.QPushButton("Abort")
+        self.abortB.hide()
         self.checkbox = qt.QCheckBox("Apply to whole dataset")
         self.checkbox.setChecked(True)
         self.checkbox.hide()
@@ -242,6 +254,7 @@ class _InputWidget(qt.QWidget):
         layout.addWidget(self.dxLE, 1, 1)
         layout.addWidget(self.dyLE, 2, 1)
         layout.addWidget(self.correctionB, 4, 0, 1, 2)
+        layout.addWidget(self.abortB, 4, 0, 1, 2)
         layout.addWidget(self.checkbox, 3, 1)
 
         self.setLayout(layout)
